# -*- coding: utf-8 -*-
from decimal import Decimal
from ringo.views.base import update as update_, list_
from ringo.lib.message import Mailer, Mail
from ringo.lib.helpers.appinfo import get_app_name
from efa.model.beneficiary import Beneficiary
from ringo_target.model import Target
from ringo_target.lib.detectsystem import get_startyear, get_totaltarget

subject_fmt = (u"Das %s-Vorhaben %s hat den Zielwert des Jahres %d auf"
               u" %d geändert.")
mailbody_fmt = u"""Vom %s-Server:
Das Vorhaben %s hat den Zielwert des Jahres %d von %d auf %d geändert."""


class MailCaller(object):
    def __init__(self, recipient, old_value):
        self.recipient = recipient
        self.old_value = old_value

    def __call__(self, request, item):
        mailer = Mailer(request)
        app_name = get_app_name()
        beneficiary = request.db.query(Beneficiary).first().name
        subject = subject_fmt % (app_name, beneficiary, item.year, item.target)
        mailbody = mailbody_fmt % (app_name, beneficiary, item.year,
                                   self.old_value, item.target)

        mail = Mail([self.recipient], subject, msg=mailbody)
        if self.old_value != item.target:
            mailer.send(mail)

        return item


def update(request):
    recipient = request.registry.settings.get("ringo_target.mail.recipient")
    if request.POST and 'year' in request.params and recipient:
 
        c_year = request.params['year']
        old_value = request.db.query(Target).filter(
                Target.year==c_year).first().target
        mail_send_callback = MailCaller(recipient, old_value)
        rvalue = update_(request, callback=mail_send_callback)
    else:
        rvalue = update_(request)
    return rvalue


def list__(request):
    rvalue = list_(request)
    request.session["%s.backurl" % Target] = request.url
    request.session.save()
    rvalue.update(create_diff_string(request))
    return rvalue


def create_diff_string(request):
    _ = request.translate
    rvalue = {}
    startyear = get_startyear(request)
    target_list = request.db.query(Target).filter(Target.year >= startyear).all()
    target_sum = Decimal(sum([obj.target for obj in target_list]))

    target_total = get_totaltarget(request)

    rvalue["total_warn"] = cmp(target_sum, target_total)
    rvalue["total_str"] = (_("sum: %s (totaltarget: %s)") % (
                           str(target_sum), str(target_total)))
    rvalue["total_value"] = target_total
    return rvalue
